<?php

/*************************************************************************\
 
  discussion.class.php - a php class for displaying a discussion board
  Copyright (C) 2001 - Paul Gareau <paul@xhawk.net>
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
\*************************************************************************/

class Discussion
{
	function Discussion($table_name='board', $board_id=1, $use_cookies=0)
	{

		/////////////////////////////////////////////////////////////
		///// you must enter values for the following variables /////
		/////////////////////////////////////////////////////////////

		// your database connnection info
		$db_type    = 'mysql'; // (string)  mysql, pgsql, ibase, msql, mssql, oci8, odbc, sybase, ifx, fbsql
		$db_persist = 1;       // (boolean) persistant db connection

		$db_host = 'localhost';
		$db_user = '';
		$db_pass = '';
		$db_name = '';

		// directory that contains discussion*.inc.php
		$config_dir = dirname(__FILE__).'/';
		// directory that contains cache files
		$cache_dir = dirname(__FILE__).'/cache/';

		// all other settings are in discussion*.inc.php

		////////////////////////////////////////////////////////////////
		///// you shouldnt have to worry about anything below here /////
		////////////////////////////////////////////////////////////////

		$this->version     = '20010820';
		$this->table_name  = $table_name;
		$this->board_id    = $board_id;
		$this->use_cookies = $use_cookies;
		$this->cache_dir   = $cache_dir;

		// if a table specific config file exists, use it
		if(file_exists("{$config_dir}discussion_{$this->table_name}.inc.php")) {
			include_once("{$config_dir}discussion_{$this->table_name}.inc.php");
		} else {
			include_once("{$config_dir}discussion.inc.php");
		}
		// set values from config file
		$this->max_indent     = $max_indent;      // (integer) max times to indent children
		$this->max_age        = $max_age;         // (integer) max age in minutes before refreshing cache file
		$this->num_posts      = $num_posts;       // (integer) number of posts per page
		$this->mail_reply     = $mail_reply;      // (boolean) 1 mails previous posters of same thread
		$this->dup_check      = $dup_check;       // (boolean) 1 enables duplicate post checking
		$this->use_ubb_code   = $use_ubb_code;    // (boolean) 1 uses ubb code for posts
		$this->use_javascript = $use_javascript;  // (boolean) 1 enables javascript
		$this->censor         = $censor;          // (boolean) 1 enables post censoring
		$this->use_caching    = $use_caching;     // (boolean) 1 enables list caching
		$this->admin          = $admin;           // (boolean) enable admin features
		$this->order_parents  = $order_parents;   // (string) order parents are displayed (ASC or DESC)
		$this->style          = $style;           // (string) flat, basic, threaded or nested
		$this->border_width   = $border_width;    // (int) table border width
		$this->err_color      = $err_color;	  // (string) error font color
		$this->new_color      = $new_color;	  // (string) error font color
		$this->th_color       = $th_color;        // (string) header bgcolor
		$this->form_color     = $form_color;      // (string) form bgcolor
		$this->post_color     = $post_color;      // (string) post bgcolor
		$this->font1          = $font1;           // (string) header font
		$this->font2          = $font2;           // (string) content font
		$this->cfont1         = $cfont1;          // (string) close header font
		$this->cfont2         = $cfont2;          // (string) close content font

		if($this->use_cookies) {
			// give each board a cookie whose name is based on its table name and board id
			// prevents multi table/board interference
			$this->cookie_read_name  = 'cookie_read_ids_'.$this->table_name;
			$this->cookie_lsession_name = 'cookie_last_session_'.$this->table_name.'_'.$this->board_id;
			$cookie_last_load = 'cookie_last_load_'.$this->table_name.'_'.$this->board_id;
			$cookie_session = 'cookie_session_'.$this->table_name.'_'.$this->board_id;

			global ${$this->cookie_lsession_name}, ${$cookie_last_load}, ${$cookie_session};

			// the last session should be set to the last page load if
			// a new session is being started, or 0 if its a first view
			if(! ${$cookie_session}) {
				if(! ${$cookie_last_load}) {
					${$cookie_last_load} = 0;
				}
				${$this->cookie_lsession_name} = ${$cookie_last_load};
				setcookie($this->cookie_lsession_name, ${$this->cookie_lsession_name});
			}
			setcookie($cookie_session, 1);
			setcookie($cookie_last_load, time(), time()+518400);
		}
		//global $DB;
		//$this->db = $DB;
		include_once 'DB.php';
		$this->db = DB::connect("$db_type://$db_user:$db_pass@$db_host/$db_name", $db_persist);
		$this->db->setFetchMode(DB_FETCHMODE_ASSOC);

		if (DB::isError($this->db)) {
			die($this->db->getMessage());
		}
	}

	/**
	 * Retrieve posts and generate html according to $this->style to display them.
	 *
	 * @param 	$offset (int) show this post first
	 * @return 	$html (string) the html to display discussion board
	 * @author 	Paul Gareau <paul@xhawk.net>
	*/

	function getTopics($offset=0)
	{
		global $PHP_SELF;

		if(! $offset) $offset = 0; // $offset may be set but empty

		$debug = 0;   //print caching actions
		$cache_file = "$this->table_name-$this->board_id-$offset.cache";

		if($this->use_caching) {
			if(file_exists($this->cache_dir.$cache_file)) {
				if(filemtime($this->cache_dir.$cache_file)>(time()-(60*$this->max_age))) {
					if($debug) echo 'using cached copy<br>';
					$fp = fopen($this->cache_dir.$cache_file, r);
					$html = fread($fp, filesize($this->cache_dir.$cache_file));
					fclose($fp);
					return $html;
				} else {
					if($debug) echo 'cache file expired - deleting<br>';
					unlink($this->cache_dir.$cache_file);
				}
			}
		}
		$sql = "SELECT count(id) FROM $this->table_name
		       WHERE board_id=$this->board_id AND thread_id=0";
		$num_total = $this->db->getOne($sql);

		$sql = "SELECT id, date, name, email, subject, body
		       FROM $this->table_name
		       WHERE board_id=$this->board_id
		       AND thread_id=0
		       ORDER BY date DESC";
		$result = $this->db->query($sql);

		if($this->style=='basic' || $this->style=='flat') {
			$html.= "<ul>\n";
		}
		while(is_array($record = $result->fetchRow())) {
			if(++$res>$offset && $rec+1<=$this->num_posts) {
				++$rec;
				$id      = $record['id'];
				$subject = $record['subject'];
				$name    = $record['name'];
				$email   = $record['email'];
				$date    = $record['date'];
				$body    = $record['body'];

				switch($this->style) {
				case 'basic':
					$html.= $this->linkHTML($id, $subject, $name, $email, $date);
					$kids = $this->getKids($id);
					$html.= ($kids) ? $kids : '<br>';
					break;
				case 'flat':
					$html.= $this->linkHTML($id, $subject, $name, $email, $date);
					break;
				case 'nested':
				case 'threaded':
					$html.= $this->postHTML($id, $subject, $name, $email, $date, $body);
					$kids = $this->getKids($id);
					$html.= ($kids) ? $kids : '<br>';
					break;
				}
			}
		}
		$result->free();
		if($this->style=='basic' || $this->style=='flat') {
			$html.= "</ul>\n";
		}
		$html.= $this->pageNavHTML($offset, $this->num_posts, $num_total);
		$html.= "$this->font2";
		$html.= strrev(">tnof/<];psbn&>a/<noissucsid>'/noissucsid/ten.kwahx//:ptth'=ferh a< yb derewoP;psbn&[>'1'=ezis tnof<");
		$html.= "$this->cfont2\n";

		if($this->use_caching) {
			if($debug) echo 'caching html<br>';
			$fp = fopen($this->cache_dir.$cache_file, w);
			fwrite($fp, ereg_replace("[\r\n]", '', $html));
			fclose($fp);
		}
		return $html;
	}

	/**
	 * Recursively find follow-ups to each post and generate html according 
	 * to $this->style to display them.
	 *
	 * @param 	$parent_id (int) id of the post to start at when getting kids
	 * @return	$html (string) to display children of a post
	 * @author	Paul Gareau <paul@xhawk.net>
	 */

	function getKids($parent_id)
	{
		global $PHP_SELF;

		$sql = "SELECT id, name, email, date, subject, body
		       FROM $this->table_name
		       WHERE parent_id=$parent_id
		       ORDER BY date ASC";
		$result = $this->db->query($sql);

		while(is_array($record = $result->fetchRow())) {
			$id      = $record['id'];
			$subject = $record['subject'];
			$name    = $record['name'];
			$email   = $record['email'];
			$date    = $record['date'];
			$body    = $record['body'];


			switch($this->style) {
			case 'threaded':
			case 'basic':
			case 'flat':
				$html.= $this->linkHTML($id, $subject, $name, $email, $date);
				break;
			case 'nested':
				$html.= $this->postHTML($id, $subject, $name, $email, $date, $body);
				$html.= '<br>';
				break;
			}
			++$this->depth;
			$html.= $this->getKids($id);
			--$this->depth;
		}
		$result->free();
		if($html && $this->depth < $this->max_indent) {
			if($this->style=='nested') {
				$width = ($this->depth) ? $this->depth*20 : 20;
				$html= "<table border=0 cellpadding=0 cellspacing=0><tr>\n\t<td width=$width>&nbsp;</td>\n\t<td>$html</td>\n\t</tr></table>\n";
			} else {
				$html = "<ul>\n$html</ul>\n";
			}
		}
		return $html;
	}

	/**
	 * Get and display an individual post and its children according to $this->style
	 *
	 * @param	$id (int) the id of the post to display
	 * @return	$html (string) the html to display the nav, post, kids and form
	 * @author	Paul Gareau <paul@xhawk.net>
	 */

	function getPost($id)
	{
		global $PHP_SELF, ${$this->cookie_read_name};

		$sql = "SELECT id, parent_id, thread_id, date, name, email, subject, body
		       FROM $this->table_name
		       WHERE id=$id";
		$result = $this->db->query($sql);

		$record = $result->fetchRow();
		$result->free();

		if(is_array($record)) {
			$id        = $record['id'];
			$parent_id = $record['parent_id'];
			$thread_id = $record['thread_id'];
			$subject   = $record['subject'];
			$name      = $record['name'];
			$email     = $record['email'];
			$date      = $record['date'];
			$body      = $record['body'];

			// mark posts as read only if post is read explicitly
			// posts will not be marked read if viewed in threaded or nested style
			if($this->use_cookies) {
				$viewed_ids = explode('.', ${$this->cookie_read_name});

				if(! in_array($id, $viewed_ids)) {
					${$this->cookie_read_name}.=$id.'.';
					setcookie($this->cookie_read_name, ${$this->cookie_read_name});
				}
			}
			$html.= $this->navHTML($thread_id, $parent_id);
			$html.= $this->postHTML($id, $subject, $name, $email, $date, $body);

			$kids = $this->getKids($id);
			$html.= ($kids) ? $kids : '<br>';

			$html.= $this->navHTML($thread_id, $parent_id);
			$subject = (! strstr($subject, 'Re:')) ? 'Re: '.$subject : $subject;

			// formHTML() needs these values to be set
			// it is called on the implementation page
			$this->post_id = $id;
			$this->post_thread_id = $thread_id;
			$this->post_subject = $subject;

			return $html;
		} else {
			$this->error = 'Post not found!';
			return false;
		}
	}

	/**
	 * Generate html to display nav links for a post
	 *
	 * @param	$thread_id (int) the thread being viewed
	 * @param	$parent_id (int) the parent of the thread being viewed
	 * @return	$html (string) the html to display the nav links
	 * @author	Paul Gareau <paul@xhawk.net>
	 */

	function navHTML($thread_id, $parent_id)
	{
		global $PHP_SELF;

		$html.= "$this->font2\n";
		$html.= "<a href='$PHP_SELF?$this->xtra_args'><b>List</b></a> : \n";
		$html.= "<a href='$PHP_SELF?view=$thread_id&$this->xtra_args'><b>Top</b></a> : \n";
		$html.= "<a href='$PHP_SELF?view=$parent_id&$this->xtra_args'><b>Parent</b></a> : \n";
		$html.= "<a href='#reply'><b>Reply</b></a>\n";
		$html.= "$this->cfont2\n";
		$html.= "<br><br>\n";
		return $html;
	}

	/**
	 * Generate html to display add new link
	 *
	 * @return	$html (string) the html to display the link
	 * @author	Paul Gareau <paul@xhawk.net>
	 */

	function addNewLinkHTML($text='New Topic')
	{
		global $PHP_SELF;

		$html.= "$this->font2\n";
		$html.= "<a href='$PHP_SELF?show=form&$this->xtra_args'><b>$text</b></a>\n";
		$html.= "$this->cfont2\n";
		$html.= "<br><br>\n";
		return $html;
	}

	/**
	 * Generate html to display navigation for threads
	 *
	 * @param	$offset (int) the post to start at
	 * @param	$limit (int) how many posts to show starting at $offset
	 * @param	$num_rows_all (int) how many parent posts (threads) are in the table
	 * @return	$html (string) the html to display the nav links
	 * @author	Paul Gareau <paul@xhawk.net>
	 */

	function pageNavHTML($offset, $limit, $num_rows_all)
	{
		global $PHP_SELF;

		$link_args = $this->xtra_args;
		$num_pages = ceil($num_rows_all/$limit);

		if($num_pages > 1) {
			$offset_back  = $offset-$limit;
			$offset_next  = $offset+$limit;
			$current_page = $offset/$limit;

			$html.= $this->font2;
			$html.= "<b>Navigation:</b>&nbsp;[";

			if($offset_back >= 0) {
				$html.="&nbsp;<b><a href='$PHP_SELF?$link_args&offset=$offset_back'>Back</a></b>";
			}
			for($page=0; $page<$num_pages; ++$page) {
				if( ($current_page-5) < $page && ($current_page+5) > $page ) {
					$offset_page = $page * $limit;
					$page_num = $page+1;

					if($current_page==$page) {
						$html.="&nbsp;<b>$page_num</b>";
					} else {
						$html.="&nbsp;<a href='$PHP_SELF?$link_args&offset=$offset_page'>$page_num</a>";
					}
				}
			}
			if( $offset_next < $num_rows_all ) {
				$html.="&nbsp;<b><a href='$PHP_SELF?$link_args&offset=$offset_next'>Next</a></b>";
			}
			$html.="&nbsp;]\n";
			$html.= "$this->cfont2\n";
			$html.="<br>\n";

			return $html;
		} else {
			return false;
		}
	}

	/**
	 * Generate html to display a bulleted link to a post
	 *
	 * @param	$id (int) the id of the post
	 * @param	$subject (string) the subject of the post
	 * @param	$name (string) name of the poster
	 * @param	$email (string) the email address of the poster
	 * @param	$date (string) the date of the post
	 * @return	$html (string) the html to display the link
	 * @author  Paul Gareau <paul@xhawk.net>
	 */

	function linkHTML($id, $subject, $name, $email, $date)
	{
		global $PHP_SELF, ${$this->cookie_read_name}, ${$this->cookie_lsession_name};

		if($email) $name = "<a href='mailto:$email'>$name</a>";

		$time_stamp = strtotime($date);
		$date = date("F jS \a\\t g A", $time_stamp);

		$html.= "<li>$this->font2<b><a href='$PHP_SELF?view=$id&$this->xtra_args'>$subject</a></b>";

		if($this->use_cookies) {
			$viewed_ids = explode('.', ${$this->cookie_read_name});

			if(! in_array($id, $viewed_ids) && $time_stamp > ${$this->cookie_lsession_name}) {
				$html.= " <font color='$this->new_color' size=1>new</font>";
			}
		}
		$html.= " by $name on $date";

		if($this->admin) {
			$html.= " [<a href='$PHP_SELF?delete=$id&$this->xtra_args'>delete</a>]";
		}
		$html.= "$this->cfont2\n";
		return $html;
	}

	/**
	 * Generate html to display a post
	 *
	 * @param	$id (int) the id of the post
	 * @param	$subject (string) the subject of the post
	 * @param	$name (string) name of the poster
	 * @param	$email (string) the email address of the poster
	 * @param	$date (string) the date of the post
	 * @param	$body (string) the body of the post
	 * @return	$html (string) the html to display the link
	 * @author  Paul Gareau <paul@xhawk.net>
	 */

	function postHTML($id, $subject, $name, $email, $date, $body)
	{
		global $PHP_SELF, ${$this->cookie_read_name}, ${$this->cookie_lsession_name};

		if($email) {
			$name = "<a href='mailto:$email'>$name</a>";
		}
		$time_stamp = strtotime($date);
		$date = date("F jS \a\\t g A", $time_stamp);

		$html.= "<table border=0 cellpadding=$this->border_width cellspacing=0 width=500>\n";
		$html.= "<tr><td bgcolor='$this->th_color'>";
		$html.= "<table border=0 cellpadding=2 cellspacing=0 width=500>\n";
		$html.= "<tr><td bgcolor='$this->th_color'>";
		$html.= "$this->font1<b><a href='$PHP_SELF?view=$id&$this->xtra_args'>$subject</a></b>";

		if($this->use_cookies) {
			$viewed_ids = explode('.', ${$this->cookie_read_name});

			if(! in_array($id, $viewed_ids) && $time_stamp > ${$this->cookie_lsession_name}) {
				$html.= " <font color='$this->new_color' size=1>new</font>";
			}
		}
		$html.= " by $name on $date";

		if($this->admin) {
			$html.= " [<a href='$PHP_SELF?delete=$id&$this->xtra_args'>delete</a>]\n";
		}
		$html.= "$this->cfont1\n";
		$html.= "</td></tr><tr>\n";
		$html.= "<td width='100%' bgcolor='$this->post_color'>$this->font2 &nbsp;&nbsp; $body$this->cfont2</td>\n";
		$html.= "</tr></table>\n";
		$html.= "</tr></table>\n";

		return $html;
	}

	/**
	 * Generate html to display the post form
	 *
	 * @return	$html (string) the html to display the form
	 * @author  Paul Gareau <paul@xhawk.net>
	 */

	function formHTML()
	{
		global $PHP_SELF, ${$this->cookie_read_name}, $cookie_user_inf;

		// get values from object properties
		// set by getPost()
		$parent_id = $this->post_id;
		$thread_id = $this->post_thread_id;
		$subject   = $this->post_subject;

		if($parent_id) {
			$msg = 'Reply to this post';
		} else {
			$parent_id=0;
			$msg = 'Post a new message';
		}

		// try to get user's name and email from properties
		// failing that try from cookie
		if($this->name || $this->email) {
			$name = $this->name;
			$email = $this->email;
		} elseif($this->use_cookies) {
			$name  = $cookie_user_inf[0];
			$email = $cookie_user_inf[1];
		}

		// if thread_id is 0 then we are replying to a top level thread
		// the thread_id of this post should be its parents id
		if($thread_id==0) {
			$thread_id = $parent_id;
		}
		if($this->use_javascript) {
			$html.= "<script type='text/javascript'>\n";
			$html.= "function check_form(){\n";
			$html.= " if(document.dform.subject.value.length < 1){\n";
			$html.= "  alert ('You must enter a subject!')\n";
			$html.= "  return false\n";
			$html.= " }\n";
			$html.= "}\n";
			$html.= "</script>\n";
		}
		$html.= "<a name='reply'></a>\n";
		$html.= "<form name='dform' action='$PHP_SELF?$this->xtra_args' method='post'";
		$html.= ($this->use_javascript) ? " onSubmit='return check_form()'>\n" : '>\n';
		$html.= "<table border=0 cellpadding=$this->border_width cellspacing=0><tr><td bgcolor='$this->th_color'>\n";
		$html.= "<table border=0 cellpadding=3 cellspacing=0 bgcolor='$this->form_color'>\n";
		$html.= "<tr><td colspan=2 bgcolor='$this->th_color'>$this->font1<b>$msg:</b>$this->cfont1</td></tr>\n";
		$html.= "<tr><td>$this->font2 Name:$this->cfont2</td>\n";
		$html.= "<td><input type='text' size=35 name='name' value='$name' maxlength=50></td></tr>\n";
		$html.= "<tr><td>$this->font2 Email:$this->cfont2</td>\n";
		$html.= "<td><input type='text' size=35 name='email' value='$email' maxlength=50></td></tr>\n";
		$html.= "<tr><td>$this->font2 Subject:$this->cfont2</td>\n";
		$html.= "<td><input type='text' size=35 name='subject' value=\"$subject\" maxlength=100></td></tr>\n";
		$html.= "<tr><td colspan=2>$this->font2 Message:$this->cfont2</td></tr>\n";
		$html.= "<tr><td colspan=2><textarea rows=10 cols=40 name=body></textarea></td></tr>\n";

		if($this->mail_reply) {
			$html.= "<tr><td colspan=2> $this->font2 You will recieve replies to this post by email.$this->cfont2</td></tr>\n";
		}
		if($this->use_ubb_code) {
			$html.= "<tr><td colspan=2> $this->font2 UBB code is enabled in this forum.$this->cfont2</td></tr>\n";
		}
		if($this->use_caching) {
			$html.= "<tr><td colspan=2> $this->font2 Your post may not appear immediately.$this->cfont2</td></tr>\n";
		}
		$html.= "<tr><td colspan=2 align='right'>\n";
		$html.= "<input type='submit' value=' Post '>\n";
		$html.= "<input type='hidden' name='post' value='post'>\n";
		$html.= "<input type='hidden' name='parent_id' value='$parent_id'>\n";
		$html.= "<input type='hidden' name='thread_id' value='$thread_id'>\n";
		$html.= "</td>";
		$html.= "</tr>\n";
		$html.= "</table>\n";
		$html.= "</td></tr></table>\n";
		$html.= "</form>";

		$html.= "$this->font2";
		$html.= strrev(">tnof/<];psbn&>a/<noissucsid>'/noissucsid/ten.kwahx//:ptth'=ferh a< yb derewoP;psbn&[>'1'=ezis tnof<");
		$html.= $this->cfont2;

		return $html;
	}

	/**
	 * Add post to database
	 *
	 * @param	$parent_id (int) the id of the parent, if post is a follow up
	 * @param	$thread_id (int) the id of the thread, if post is a follow up
	 * @param	$subject (int) the subject, if post is a follow up
	 * @param	$name (string) name of the poster
	 * @param	$email (string) the email address of the poster
	 * @param	$subject (string) the subject of the post
	 * @param	$body (string) the body of the post
	 * @return	$html (string) the html to display the form
	 * @author  Paul Gareau <paul@xhawk.net>
	 */

	function addPost($parent_id, $thread_id, $name, $email, $subject, $body)
	{
		global $HTTP_HOST, $PHP_SELF, ${$this->cookie_read_name};

		if(! $subject) {
			$this->error = 'You must enter a subject!';
			return false;
		}
		if(! get_magic_quotes_gpc()) {
			$name    = addslashes($name);
			$email   = addslashes($email);
			$subject = addslashes($subject);
			$body    = addslashes($body);
		}

		//$name    = dbPrep($name);
		//$email   = dbPrep($email);
		//$subject = dbPrep($subject);
		//$body    = dbPrep($body);

		// general clean up of fields
		$name		= htmlentities(trim($name));
		$email      = htmlentities(trim($email));
		$subject    = htmlentities(trim($subject));
		$body       = trim($body);

		if($this->censor) {
			// illegal words
			$bad_words  = "shit|fuck|cunt|twat|pussy|asshole";
			$bw_replace = '[beep]'; //'!@#$%'; // its your choice...

			$name       = eregi_replace($bad_words, $bw_replace, $name);
			$email      = eregi_replace($bad_words, $bw_replace, $email);
			$subject    = eregi_replace($bad_words, $bw_replace, $subject);
			$body       = eregi_replace($bad_words, $bw_replace, $body);
		}
		if(! $name) $name = '[anonymous]';
		if(! $body) $body = '[no text]';

		$mail_body  = $body; //save body now for mailing
		$body       = htmlentities($body); // change html to entities BEFORE ubb2html
		$body       = str_replace("\n", '<br>', $body); // better than nl2br()

		if($this->use_ubb_code) {
			$body = $this->ubb2html($body);
		}

		///// all post modification must be done ABOVE this point! /////

		if($this->use_cookies) {
			setcookie('cookie_user_inf[0]', $name,  time()+518400);
			setcookie('cookie_user_inf[1]', $email, time()+518400);
		}
		if($this->dup_check) {
			$sql = "SELECT count(id)
			       FROM $this->table_name
			       WHERE parent_id=$parent_id
			       AND name='$name' AND email='$email' AND subject='$subject' AND body='$body'";
			$dups = $this->db->getOne($sql);

			if($dups) {
				$this->error = 'Duplicate posts are not allowed!';
				return false;
			}
		}
		$sql = "INSERT INTO $this->table_name (board_id, parent_id, thread_id, date, name, email, subject, body)
		       VALUES($this->board_id, $parent_id, $thread_id, NOW(), '$name', '$email', '$subject', '$body')";
		$result = $this->db->query($sql);

		if(!DB::isError($result)) {

			// get id of the post we just added
			$sql = "SELECT id FROM $this->table_name
			       WHERE parent_id=$parent_id
			       AND name='$name' AND email='$email' AND subject='$subject'";
			$post_id = $this->db->getOne($sql);

			// mark this post read
			if($this->use_cookies) {
				${$this->cookie_read_name}.=$post_id.'.';
				setcookie($this->cookie_read_name, ${$this->cookie_read_name});
			}
			// send if mail_reply is enabled
			// and if it is not a first post
			if($this->mail_reply && $thread_id!=0) {
				// keep slashes from appearing in email
				$mail_name    = stripslashes($name);
				$mail_subject = stripslashes($subject);
				$mail_body    = stripslashes(str_replace("\r", '', $mail_body));

				// we dont want ubb code in email either
				if($this->use_ubb_code) {
					$mail_body = $this->stripUBB($mail_body);
				}
				// format subject and body
				$mail_subject = "[discussion] ".$mail_subject;
				$mail_body    = "$mail_name says:\n\n".
				                "$mail_body\n\n\n".
				                "Reply to this message:\n".
				                "http://$HTTP_HOST$PHP_SELF?view=$post_id&$this->xtra_args";
				$mail_body   .= $this->email_append;

				$sql = "SELECT DISTINCT email FROM $this->table_name
				       WHERE (thread_id=$thread_id OR id=$thread_id)
				       AND email != '$email'";
				$result = $this->db->query($sql);

				while(is_array($record = $result->fetchRow())) {
					$to = $record['email'];

					// very simple address checking
					if(ereg(".+@.+\\..+", $to)) {
						mail($to, $mail_subject, $mail_body, "From: discussion@$HTTP_HOST");
					}
				}
			}
			return true;
		} else {
			$this->error = 'Could not add post!';
			return false;
		}
	}

	/**
	 * Delete a post and its children (recursively)
	 *
	 * @param	$id (int) id of the post to delete
	 * @return	true on success, false on falure
	 * @author  Paul Gareau <paul@xhawk.net>
	 */

	function deletePost($id)
	{
		$sql = "SELECT id FROM $this->table_name WHERE parent_id=$id";
		$result = $this->db->query($sql);

		while(is_array($record = $result->fetchRow())) {
			$this->deletePost($record['id']);
		}
		$sql = "DELETE FROM $this->table_name WHERE id=$id";
		$result = $this->db->query($sql);

		if(!DB::isError($result)) {
			return true;
		} else {
			$this->error = 'Could not delete post!';
			return false;
		}
	}

	/**
	 * Convert ubb code to html
	 *
	 * @param	$string (string) the string of text to be converted
	 * @return	$string (string) the converted string
	 * @author	Paul Gareau <paul@xhawk.net>
	 */

	function ubb2html($string)
	{
		$string = preg_replace("/\[b\](.*?)\[\/b\]/si", "<b>\\1</b>", $string);
		$string = preg_replace("/\[i\](.*?)\[\/i\]/si", "<i>\\1</i>", $string);
		$string = preg_replace("/\[u\](.*?)\[\/u\]/si", "<u>\\1</u>", $string);
		$string = preg_replace("/\[p\](.*?)\[\/p\]/si", "<p>\\1</p>", $string);
		$string = preg_replace("/\[code\](.*?)\[\/code\]/si", "<blockquote><pre>\\1</pre></blockquote>", $string);
		$string = preg_replace("/\[quote\](.*?)\[\/quote\]/si", "<blockquote>\\1</blockquote>", $string);
		$string = preg_replace("/(^|\s)(http:\/\/\S+)/si", "\\1<a href=\"\\2\">\\2</a>", $string);
		$string = preg_replace("/(^|\s)(www\.\S+)/si", "\\1<a href=\"http://\\2\">\\2</a>", $string);
		$string = preg_replace("/\[url\](http|https|ftp)(:\/\/\S+?)\[\/url\]/si",
		                       "<a href=\"\\1\\2\" target=\"_blank\">\\1\\2</a>", $string);
		$string = preg_replace("/\[url\](\S+?)\[\/url\]/si",
		                       "<a href=\"http://\\1\" target=\"_blank\">\\1</a>", $string);
		$string = preg_replace("/\[url=(http|https|ftp)(:\/\/\S+?)\](.*?)\[\/url\]/si",
		                       "<a href=\"\\1\\2\" target=\"_blank\">\\3</a>", $string);
		$string = preg_replace("/\[url=(\S+?)\](\S+?)\[\/url\]/si",
		                       "<a href=\"http://\\1\" target=\"_blank\">\\2</a>", $string);
		$string = preg_replace("/\[email\](\S+?@\S+?\\.\S+?)\[\/email\]/si",
		                       "<a href=\"mailto:\\1\">\\1</a>", $string);
		$string = preg_replace("/\[img\](\S+?)\[\/img\]/si",
		                       "<img src=\"\\1\" border=0 alt=\"\\1\">", $string);
		return $string;
	}

	/**
	 * Strip ubb tags from string
	 *
	 * @param   $string (string) the string of text to be stripped
	 * @return  $string (string) the converted string
	 * @author  Paul Gareau <paul@xhawk.net>
	 */

	function stripUBB($string)
	{
		$string = eregi_replace("\[b\]", '', $string);
		$string = eregi_replace("\[/b\]", '', $string);
		$string = eregi_replace("\[i\]", '', $string);
		$string = eregi_replace("\[/i\]", '', $string);
		$string = eregi_replace("\[u\]", '', $string);
		$string = eregi_replace("\[/u\]", '', $string);
		$string = eregi_replace("\[p\]", '', $string);
		$string = eregi_replace("\[/p\]", '', $string);
		$string = eregi_replace("\[code\]", '', $string);
		$string = eregi_replace("\[/code\]", '', $string);
		$string = eregi_replace("\[quote\]", '', $string);
		$string = eregi_replace("\[/quote\]", '', $string);
		$string = preg_replace ("/\[url(\S*?)\]/si", '', $string);
		$string = eregi_replace("\[/url\]", '', $string);
		$string = eregi_replace("\[email\]", '', $string);
		$string = eregi_replace("\[/email\]", '', $string);
		$string = eregi_replace("\[img\]", '', $string);
		$string = eregi_replace("\[/img\]", '', $string);
		return $string;
	}

	/**
	 * Generate html to display an error message
	 *
	 * @param	$msg (string) error message, if blank $this->error will be used
	 * @return	$html (sring) the html to display the message
	 * @author  Paul Gareau <paul@xhawk.net>
	 */

	function error($msg='')
	{
		if(! $msg) $msg = $this->error;
		$html.= "<table><tr><td>$this->font2<font color='$this->err_color'>\n";
		$html.= "$msg\n";
		$html.= "</font></font></td></tr></table><br>\n";

		return $html;
	}
}

?>
